@@ -586,11 +586,7 @@ fribidi_analyse_string (	/* input */
   /* Determinate character types */
   DBG ("  Determine character types\n");
   {
-#ifndef _MSC_VER /* just another gccism ... */
-    FriBidiCharType char_type[len];
-#else
-    FriBidiCharType* char_type = _alloca(len*sizeof(FriBidiCharType));
-#endif
+    FriBidiCharType* char_type = g_alloca (len*sizeof(FriBidiCharType));
     for (i = 0; i < len; i++)
       char_type[i] = _pango_fribidi_get_type (str[i]);
 
