@@ -97,7 +97,7 @@ _pango_ft2_font_new (PangoFontMap    *fontmap,
   ft2font->font_pattern = pattern;
   
   g_object_ref (G_OBJECT (fontmap));
-  ft2font->description = _pango_ft2_font_desc_from_pattern (pattern);
+  ft2font->description = _pango_ft2_font_desc_from_pattern (pattern, TRUE);
   ft2font->face = NULL;
 
   if (MiniXftPatternGetDouble (pattern, XFT_PIXEL_SIZE, 0, &d) == MiniXftResultMatch)
@@ -280,7 +280,7 @@ pango_ft2_render (FT_Bitmap        *bitmap,
   int x_position = 0;
   int ix, iy, ixoff, iyoff, y_start, y_limit, x_start, x_limit;
   PangoGlyphInfo *gi;
-  guchar *p, *q;
+  guchar *dest, *src;
 
   g_return_if_fail (bitmap != NULL);
   g_return_if_fail (glyphs != NULL);
@@ -305,12 +305,11 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	      ixoff = x + PANGO_PIXELS (x_position + gi->geometry.x_offset);
 	      iyoff = y + PANGO_PIXELS (gi->geometry.y_offset);
 
-	      x_start = MAX (0, -face->glyph->bitmap_left - ixoff);
-	      x_limit = MIN (face->glyph->bitmap.width, face->glyph->bitmap_left - ixoff + bitmap->width);
-
-	      y_start = MAX (0, face->glyph->bitmap_top - iyoff);
-	      y_limit = MIN (face->glyph->bitmap.rows, face->glyph->bitmap_top - iyoff + bitmap->rows);
+	      x_start = MAX (0, - (ixoff + face->glyph->bitmap_left));
+	      x_limit = MIN (face->glyph->bitmap.width, bitmap->width - (ixoff + face->glyph->bitmap_left));
 
+	      y_start = MAX (0,  - (iyoff - face->glyph->bitmap_top));
+	      y_limit = MIN (face->glyph->bitmap.rows, bitmap->rows - (iyoff - face->glyph->bitmap_top));
 
 	      PING (("glyph %d:%d: bitmap: %dx%d, left:%d top:%d",
 		     i, glyph_index,
@@ -322,48 +321,47 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	      if (face->glyph->bitmap.pixel_mode == ft_pixel_mode_grays)
 		for (iy = y_start; iy < y_limit; iy++)
 		  {
-		    p = bitmap->buffer +
+		    dest = bitmap->buffer +
 		      (iyoff - face->glyph->bitmap_top + iy) * bitmap->pitch +
-		      ixoff + face->glyph->bitmap_left + 
-                      x_start;
+		      ixoff + face->glyph->bitmap_left + x_start;
 		    
-		    q = face->glyph->bitmap.buffer + 
-                      iy * face->glyph->bitmap.pitch;
+		    src = face->glyph->bitmap.buffer + 
+                      iy * face->glyph->bitmap.pitch + x_start;
 
 		    for (ix = x_start; ix < x_limit; ix++)
 		      {
-                        switch (*q)
+                        switch (*src)
                           {
                           case 0:
                             break;
                           case 0xff:
-                            *p = 0xff;
+                            *dest = 0xff;
                           default:
-                            *p = MIN ((gushort) *p + (gushort) *q, 0xff);
+                            *dest = MIN ((gushort) *dest + (gushort) *src, 0xff);
                             break;
                           }
-                        q++;
-			p++;
+                        dest++;
+			src++;
 		      }
 		  }
 	      else if (face->glyph->bitmap.pixel_mode == ft_pixel_mode_mono)
 		for (iy = y_start; iy < y_limit; iy++)
 		  {
-		    p = bitmap->buffer +
+		    dest = bitmap->buffer +
 		      (iyoff - face->glyph->bitmap_top + iy) * bitmap->pitch +
 		      ixoff + face->glyph->bitmap_left + 
                       x_start;
                     
-                    q = face->glyph->bitmap.buffer + 
+                    src = face->glyph->bitmap.buffer + 
                       iy*face->glyph->bitmap.pitch;
 
 		    for (ix = x_start; ix < x_limit; ix++)
 		      {
-			if ((*q) & (1 << (7 - (ix % 8))))
-			  *p = 0xff;
+			if ((*src) & (1 << (7 - (ix % 8))))
+			  *dest |= (1 << (7 - (ix % 8)));
 			if ((ix % 8) == 7)
-			  q++;
-			p++;
+			  src++;
+			dest++;
 		      }
 		  }
 	      else
@@ -573,7 +571,6 @@ pango_ft2_font_describe (PangoFont *font)
   ft2font = PANGO_FT2_FONT (font);
 
   desc = pango_font_description_copy (ft2font->description);
-  pango_font_description_set_size (desc, ft2font->size);
 
   return desc;
 }
@@ -593,6 +590,14 @@ pango_ft2_get_shaper_map (PangoLanguage *language)
   return pango_find_map (language, engine_type_id, render_type_id);
 }
 
+/**
+ * pango_ft2_font_get_coverage:
+ * @font: a #PangoFT2Font.
+ * @language: a language tag.
+ * @returns: a #PangoCoverage.
+ * 
+ * Should not be called directly, use pango_font_get_coverage() instead.
+ **/
 PangoCoverage *
 pango_ft2_font_get_coverage (PangoFont     *font,
 			     PangoLanguage *language)
@@ -734,7 +739,7 @@ pango_ft2_render_layout_line (FT_Bitmap       *bitmap,
  * @x:         the X position of the left of the layout (in pixels)
  * @y:         the Y position of the top of the layout (in pixels)
  *
- * Render a #PangoLayoutLine onto a FreeType2 bitmap
+ * Render a #PangoLayout onto a FreeType2 bitmap
  */
 void 
 pango_ft2_render_layout (FT_Bitmap   *bitmap,
@@ -749,7 +754,7 @@ pango_ft2_render_layout (FT_Bitmap   *bitmap,
   int width;
   int y_offset = 0;
 
-  gboolean first = FALSE;
+  gboolean first = TRUE;
   
   g_return_if_fail (bitmap != NULL);
   g_return_if_fail (layout != NULL);
