@@ -125,7 +125,7 @@ _pango_xft_font_new (PangoFontMap               *fontmap,
   xfont->font_pattern = pattern;
   
   g_object_ref (G_OBJECT (fontmap));
-  xfont->description = _pango_xft_font_desc_from_pattern (pattern);
+  xfont->description = _pango_xft_font_desc_from_pattern (pattern, TRUE);
   xfont->xft_font = NULL;
 
   _pango_xft_font_map_add (xfont->fontmap, xfont);
@@ -225,14 +225,14 @@ draw_box (Display      *display,
 
 /**
  * pango_xft_render:
- * @draw:    the XftDraw object.
+ * @draw:    the <type>XftDraw</type> object.
  * @color:   the color in which to draw the string
  * @font:    the font in which to draw the string
  * @glyphs:  the glyph string to draw
  * @x:       the x position of start of string (in pixels)
  * @y:       the y position of baseline (in pixels)
  *
- * Render a PangoGlyphString onto an XftDraw object wrapping an X drawable.
+ * Renders a #PangoGlyphString onto an <type>XftDraw</type> object wrapping an X drawable.
  */
 static void
 pango_xft_real_render (Display          *display,
@@ -326,14 +326,14 @@ pango_xft_real_render (Display          *display,
 
 /**
  * pango_xft_render:
- * @draw:    the XftDraw object.
+ * @draw:    the <type>XftDraw</type> object.
  * @color:   the color in which to draw the string
  * @font:    the font in which to draw the string
  * @glyphs:  the glyph string to draw
  * @x:       the x position of start of string (in pixels)
  * @y:       the y position of baseline (in pixels)
  *
- * Render a PangoGlyphString onto an XftDraw object wrapping an X drawable.
+ * Renders a #PangoGlyphString onto an <type>XftDraw</type> object wrapping an X drawable.
  */
 void
 pango_xft_render (XftDraw          *draw,
@@ -352,8 +352,8 @@ pango_xft_render (XftDraw          *draw,
 }
 
 /**
- * pango_xft_picture render:
- * @draw:         an X display
+ * pango_xft_picture_render:
+ * @display:      an X display
  * @src_picture:  the source picture to draw the string with
  * @dest_picture: the destination picture to draw the strign onto
  * @font:         the font in which to draw the string
@@ -361,7 +361,7 @@ pango_xft_render (XftDraw          *draw,
  * @x:            the x position of start of string (in pixels)
  * @y:            the y position of baseline (in pixels)
  *
- * Render a PangoGlyphString onto an Xrender picture object.
+ * Renders a #PangoGlyphString onto an Xrender <type>Picture</type> object.
  */
 void
 pango_xft_picture_render (Display          *display,
@@ -478,7 +478,7 @@ pango_xft_font_get_coverage (PangoFont     *font,
 	pango_coverage_set (coverage, i, PANGO_COVERAGE_EXACT);
     }
 
-  _pango_xft_font_map_set_coverage (xfont->fontmap, g_strdup (filename), coverage);
+  _pango_xft_font_map_set_coverage (xfont->fontmap, filename, coverage);
       
   return coverage;
 }
